<?php

namespace Drupal\Tests\field_fallback\Functional;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\field\Traits\EntityReferenceFieldCreationTrait;
use Drupal\Tests\field_ui\Traits\FieldUiTestTrait;
use Drupal\Tests\taxonomy\Traits\TaxonomyTestTrait;
use Drupal\field\Entity\FieldConfig;
use Drupal\taxonomy\VocabularyInterface;

/**
 * Test field_fallback with entity references.
 *
 * @group field_fallback
 */
class FieldFallbackEntityReferenceTest extends BrowserTestBase {

  use EntityReferenceFieldCreationTrait;
  use FieldUiTestTrait;
  use TaxonomyTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field_fallback_test',
    'block',
    'field_ui',
    'node',
    'taxonomy',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'starterkit_theme';

  /**
   * An taxonomy vocabulary.
   *
   * @var \Drupal\taxonomy\VocabularyInterface
   */
  protected VocabularyInterface $vocabulary;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->drupalPlaceBlock('system_breadcrumb_block');
    $this->drupalCreateContentType(['type' => 'page']);

    $this->drupalLogin($this->rootUser);

    $this->vocabulary = $this->createVocabulary();
    $this->createEntityReferenceField('node', 'page', 'field_primary_term', 'Primary term', 'taxonomy_term', 'default', ['target_bundles' => [$this->vocabulary->id()]]);
    $this->createEntityReferenceField('node', 'page', 'field_secondary_term', 'Secondary term', 'taxonomy_term', 'default', ['target_bundles' => [$this->vocabulary->id()]]);

    $field_config = FieldConfig::loadByName('node', 'page', 'field_secondary_term');
    $field_config->setThirdPartySetting('field_fallback', 'field', 'field_primary_term');
    $field_config->setThirdPartySetting('field_fallback', 'converter', 'default');
    $field_config->save();

    // Add fields to the form display.
    EntityFormDisplay::load('node.page.default')
      ->setComponent('field_primary_term', ['type' => 'options_select'])
      ->setComponent('field_secondary_term', ['type' => 'options_select'])
      ->save();

    // This rebuild is necessary or the tests won't know about our newly
    // created fields.
    $this->rebuildAll();
  }

  /**
   * Test field fallback with entity references.
   */
  public function testEntityReferenceValues(): void {
    $term = $this->createTerm($this->vocabulary);
    $this->drupalGet('node/add/page');
    $title = $this->randomMachineName();
    $this->submitForm([
      'title[0][value]' => $title,
    ], 'Save');

    // Check if the value is empty when no entity was selected..
    $node = $this->drupalGetNodeByTitle($title, TRUE);
    $this->assertTrue($node->get('field_secondary_term')->isEmpty());

    $this->drupalGet(sprintf('node/%s/edit', $node->id()));
    $this->submitForm([
      'field_primary_term' => $term->id(),
    ], 'Save');

    // Check that the value matches the chosen taxonomy term.
    $node = $this->drupalGetNodeByTitle($title, TRUE);
    $this->assertEquals($term->id(), $node->get('field_secondary_term')->target_id);

    $this->drupalGet(sprintf('node/%s/edit', $node->id()));
    $this->submitForm([
      'title[0][value]' => $title,
      'field_primary_term' => '_none',
    ], 'Save');

    // Check that the value is empty.
    $node = $this->drupalGetNodeByTitle($title, TRUE);
    $this->assertTrue($node->get('field_secondary_term')->isEmpty());
  }

}
